@@ -6,6 +6,7 @@
 #![allow(rustdoc::private_intra_doc_links)]
 
 use std::cmp::Ordering;
+use std::ffi::OsString;
 use std::io::{self, BufReader, ErrorKind};
 use std::{
     fs::{File, remove_file},
@@ -16,7 +17,7 @@ use clap::{Arg, ArgAction, ArgMatches, Command};
 use regex::Regex;
 use uucore::display::Quotable;
 use uucore::error::{FromIo, UResult};
-use uucore::{format_usage, help_about, help_section, help_usage};
+use uucore::format_usage;
 
 mod csplit_error;
 mod patterns;
@@ -25,9 +26,7 @@ mod split_name;
 use crate::csplit_error::CsplitError;
 use crate::split_name::SplitName;
 
-const ABOUT: &str = help_about!("csplit.md");
-const AFTER_HELP: &str = help_section!("after help", "csplit.md");
-const USAGE: &str = help_usage!("csplit.md");
+use uucore::translate;
 
 mod options {
     pub const SUFFIX_FORMAT: &str = "suffix-format";
@@ -87,7 +86,7 @@ impl<T: BufRead> Iterator for LinesWithNewlines<T> {
     fn next(&mut self) -> Option<Self::Item> {
         fn ret(v: Vec<u8>) -> io::Result<String> {
             String::from_utf8(v).map_err(|_| {
-                io::Error::new(ErrorKind::InvalidData, "stream did not contain valid UTF-8")
+                io::Error::new(ErrorKind::InvalidData, translate!("csplit-stream-not-utf8"))
             })
         }
 
@@ -117,23 +116,32 @@ where
     T: BufRead,
 {
     let enumerated_input_lines = LinesWithNewlines::new(input)
-        .map(|line| line.map_err_context(|| "read error".to_string()))
+        .map(|line| line.map_err_context(|| translate!("csplit-read-error")))
         .enumerate();
     let mut input_iter = InputSplitter::new(enumerated_input_lines);
     let mut split_writer = SplitWriter::new(options);
-    let patterns: Vec<patterns::Pattern> = patterns::get_patterns(patterns)?;
-    let ret = do_csplit(&mut split_writer, patterns, &mut input_iter);
+    let patterns_vec: Vec<patterns::Pattern> = patterns::get_patterns(patterns)?;
+    let all_up_to_line = patterns_vec
+        .iter()
+        .all(|p| matches!(p, patterns::Pattern::UpToLine(_, _)));
+    let ret = do_csplit(&mut split_writer, patterns_vec, &mut input_iter);
 
     // consume the rest, unless there was an error
     if ret.is_ok() {
         input_iter.rewind_buffer();
         if let Some((_, line)) = input_iter.next() {
+            // There is remaining input: create a final split and copy remainder
             split_writer.new_writer()?;
             split_writer.writeln(&line?)?;
             for (_, line) in input_iter {
                 split_writer.writeln(&line?)?;
             }
             split_writer.finish_split();
+        } else if all_up_to_line && options.suppress_matched {
+            // GNU semantics for integer patterns with --suppress-matched:
+            // even if no remaining input, create a final (possibly empty) split
+            split_writer.new_writer()?;
+            split_writer.finish_split();
         }
     }
     // delete files on error by default
@@ -164,7 +172,7 @@ where
                         // the error happened when applying the pattern more than once
                         Err(CsplitError::LineOutOfRange(_)) if ith != 1 => {
                             return Err(CsplitError::LineOutOfRangeOnRepetition(
-                                pattern_as_str.to_string(),
+                                pattern_as_str,
                                 ith - 1,
                             ));
                         }
@@ -196,17 +204,17 @@ where
                         // the error happened when applying the pattern more than once
                         (Err(CsplitError::MatchNotFound(_)), Some(m)) if m != 1 && ith != 1 => {
                             return Err(CsplitError::MatchNotFoundOnRepetition(
-                                pattern_as_str.to_string(),
+                                pattern_as_str,
                                 ith - 1,
                             ));
                         }
                         (Err(err), _) => return Err(err),
                         // continue the splitting process
                         (Ok(()), _) => (),
-                    };
+                    }
                 }
             }
-        };
+        }
     }
     Ok(())
 }
@@ -241,7 +249,7 @@ impl Drop for SplitWriter<'_> {
 }
 
 impl SplitWriter<'_> {
-    fn new(options: &CsplitOptions) -> SplitWriter {
+    fn new(options: &CsplitOptions) -> SplitWriter<'_> {
         SplitWriter {
             options,
             counter: 0,
@@ -272,7 +280,7 @@ impl SplitWriter<'_> {
     }
 
     /// Writes the line to the current split.
-    /// If [`self.dev_null`] is true, then the line is discarded.
+    /// If `self.dev_null` is true, then the line is discarded.
     ///
     /// # Errors
     ///
@@ -285,7 +293,7 @@ impl SplitWriter<'_> {
                     current_writer.write_all(bytes)?;
                     self.size += bytes.len();
                 }
-                None => panic!("trying to write to a split that was not created"),
+                None => panic!("{}", translate!("csplit-write-split-not-created")),
             }
         }
         Ok(())
@@ -428,7 +436,7 @@ impl SplitWriter<'_> {
                             self.writeln(&line)?;
                         }
                         _ => (),
-                    };
+                    }
                     offset -= 1;
 
                     // write the extra lines required by the offset
@@ -443,7 +451,7 @@ impl SplitWriter<'_> {
                                     pattern_as_str.to_string(),
                                 ));
                             }
-                        };
+                        }
                         offset -= 1;
                     }
                     self.finish_split();
@@ -564,7 +572,7 @@ where
         self.size = size;
     }
 
-    /// Add a line to the buffer. If the buffer has [`self.size`] elements, then its head is removed and
+    /// Add a line to the buffer. If the buffer has `self.size` elements, then its head is removed and
     /// the new line is pushed to the buffer. The removed head is then available in the returned
     /// option.
     fn add_line_to_buffer(&mut self, ln: usize, line: String) -> Option<String> {
@@ -606,16 +614,16 @@ where
 
 #[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
-    let matches = uu_app().try_get_matches_from(args)?;
+    let matches = uucore::clap_localization::handle_clap_result(uu_app(), args)?;
 
     // get the file to split
-    let file_name = matches.get_one::<String>(options::FILE).unwrap();
+    let file_name = matches.get_one::<OsString>(options::FILE).unwrap();
 
     // get the patterns to split on
     let patterns: Vec<String> = matches
         .get_many::<String>(options::PATTERN)
         .unwrap()
-        .map(|s| s.to_string())
+        .map(ToOwned::to_owned)
         .collect();
     let options = CsplitOptions::new(&matches)?;
     if file_name == "-" {
@@ -631,8 +639,9 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
 pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
         .version(uucore::crate_version!())
-        .about(ABOUT)
-        .override_usage(format_usage(USAGE))
+        .help_template(uucore::localized_help_template(uucore::util_name()))
+        .about(translate!("csplit-about"))
+        .override_usage(format_usage(&translate!("csplit-usage")))
         .args_override_self(true)
         .infer_long_args(true)
         .arg(
@@ -640,26 +649,26 @@ pub fn uu_app() -> Command {
                 .short('b')
                 .long(options::SUFFIX_FORMAT)
                 .value_name("FORMAT")
-                .help("use sprintf FORMAT instead of %02d"),
+                .help(translate!("csplit-help-suffix-format")),
         )
         .arg(
             Arg::new(options::PREFIX)
                 .short('f')
                 .long(options::PREFIX)
                 .value_name("PREFIX")
-                .help("use PREFIX instead of 'xx'"),
+                .help(translate!("csplit-help-prefix")),
         )
         .arg(
             Arg::new(options::KEEP_FILES)
                 .short('k')
                 .long(options::KEEP_FILES)
-                .help("do not remove output files on errors")
+                .help(translate!("csplit-help-keep-files"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::SUPPRESS_MATCHED)
                 .long(options::SUPPRESS_MATCHED)
-                .help("suppress the lines matching PATTERN")
+                .help(translate!("csplit-help-suppress-matched"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
@@ -667,7 +676,7 @@ pub fn uu_app() -> Command {
                 .short('n')
                 .long(options::DIGITS)
                 .value_name("DIGITS")
-                .help("use specified number of digits instead of 2"),
+                .help(translate!("csplit-help-digits")),
         )
         .arg(
             Arg::new(options::QUIET)
@@ -675,21 +684,22 @@ pub fn uu_app() -> Command {
                 .long(options::QUIET)
                 .visible_short_alias('s')
                 .visible_alias("silent")
-                .help("do not print counts of output file sizes")
+                .help(translate!("csplit-help-quiet"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::ELIDE_EMPTY_FILES)
                 .short('z')
                 .long(options::ELIDE_EMPTY_FILES)
-                .help("remove empty output files")
+                .help(translate!("csplit-help-elide-empty-files"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::FILE)
                 .hide(true)
                 .required(true)
-                .value_hint(clap::ValueHint::FilePath),
+                .value_hint(clap::ValueHint::FilePath)
+                .value_parser(clap::value_parser!(OsString)),
         )
         .arg(
             Arg::new(options::PATTERN)
@@ -697,7 +707,7 @@ pub fn uu_app() -> Command {
                 .action(ArgAction::Append)
                 .required(true),
         )
-        .after_help(AFTER_HELP)
+        .after_help(translate!("csplit-after-help"))
 }
 
 #[cfg(test)]
@@ -725,7 +735,7 @@ mod tests {
                 assert_eq!(input_splitter.buffer_len(), 1);
             }
             item => panic!("wrong item: {item:?}"),
-        };
+        }
 
         match input_splitter.next() {
             Some((1, Ok(line))) => {
@@ -734,7 +744,7 @@ mod tests {
                 assert_eq!(input_splitter.buffer_len(), 2);
             }
             item => panic!("wrong item: {item:?}"),
-        };
+        }
 
         match input_splitter.next() {
             Some((2, Ok(line))) => {
@@ -746,7 +756,7 @@ mod tests {
                 assert_eq!(input_splitter.buffer_len(), 2);
             }
             item => panic!("wrong item: {item:?}"),
-        };
+        }
 
         input_splitter.rewind_buffer();
 
@@ -756,7 +766,7 @@ mod tests {
                 assert_eq!(input_splitter.buffer_len(), 1);
             }
             item => panic!("wrong item: {item:?}"),
-        };
+        }
 
         match input_splitter.next() {
             Some((2, Ok(line))) => {
@@ -764,7 +774,7 @@ mod tests {
                 assert_eq!(input_splitter.buffer_len(), 0);
             }
             item => panic!("wrong item: {item:?}"),
-        };
+        }
 
         match input_splitter.next() {
             Some((3, Ok(line))) => {
@@ -772,7 +782,7 @@ mod tests {
                 assert_eq!(input_splitter.buffer_len(), 0);
             }
             item => panic!("wrong item: {item:?}"),
-        };
+        }
 
         assert!(input_splitter.next().is_none());
     }
@@ -798,7 +808,7 @@ mod tests {
                 assert_eq!(input_splitter.buffer_len(), 1);
             }
             item => panic!("wrong item: {item:?}"),
-        };
+        }
 
         match input_splitter.next() {
             Some((1, Ok(line))) => {
@@ -807,7 +817,7 @@ mod tests {
                 assert_eq!(input_splitter.buffer_len(), 2);
             }
             item => panic!("wrong item: {item:?}"),
-        };
+        }
 
         match input_splitter.next() {
             Some((2, Ok(line))) => {
@@ -816,7 +826,7 @@ mod tests {
                 assert_eq!(input_splitter.buffer_len(), 3);
             }
             item => panic!("wrong item: {item:?}"),
-        };
+        }
 
         input_splitter.rewind_buffer();
 
@@ -827,7 +837,7 @@ mod tests {
                 assert_eq!(input_splitter.buffer_len(), 3);
             }
             item => panic!("wrong item: {item:?}"),
-        };
+        }
 
         match input_splitter.next() {
             Some((0, Ok(line))) => {
@@ -835,7 +845,7 @@ mod tests {
                 assert_eq!(input_splitter.buffer_len(), 2);
             }
             item => panic!("wrong item: {item:?}"),
-        };
+        }
 
         match input_splitter.next() {
             Some((1, Ok(line))) => {
@@ -843,7 +853,7 @@ mod tests {
                 assert_eq!(input_splitter.buffer_len(), 1);
             }
             item => panic!("wrong item: {item:?}"),
-        };
+        }
 
         match input_splitter.next() {
             Some((2, Ok(line))) => {
@@ -851,7 +861,7 @@ mod tests {
                 assert_eq!(input_splitter.buffer_len(), 0);
             }
             item => panic!("wrong item: {item:?}"),
-        };
+        }
 
         match input_splitter.next() {
             Some((3, Ok(line))) => {
@@ -859,7 +869,7 @@ mod tests {
                 assert_eq!(input_splitter.buffer_len(), 0);
             }
             item => panic!("wrong item: {item:?}"),
-        };
+        }
 
         assert!(input_splitter.next().is_none());
     }
