@@ -3,10 +3,14 @@
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
 
-// spell-checker:ignore abcdefghijklmnopqrstuvwxyz Anone
+// spell-checker:ignore abcdefghijklmnopqrstuvwxyz Anone fdbb littl
+
+#[cfg(unix)]
+use std::io::Read;
 
-use crate::common::util::TestScenario;
 use unindent::unindent;
+use uutests::util::TestScenario;
+use uutests::{at_and_ucmd, new_ucmd, util_name};
 
 // octal dump of 'abcdefghijklmnopqrstuvwxyz\n'
 static ALPHA_OUT: &str = "
@@ -17,7 +21,7 @@ static ALPHA_OUT: &str = "
 
 #[test]
 fn test_invalid_arg() {
-    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+    new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
 }
 
 // Test that od can read one file and dump with default format
@@ -26,39 +30,30 @@ fn test_file() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
     at.write("test", "abcdefghijklmnopqrstuvwxyz\n");
-    scene
-        .ucmd()
-        .arg("--endian=little")
-        .arg("test")
-        .succeeds()
-        .no_stderr()
-        .stdout_is(unindent(ALPHA_OUT));
-    scene
-        .ucmd()
-        .arg("--endian=littl") // spell-checker:disable-line
-        .arg("test")
-        .succeeds()
-        .no_stderr()
-        .stdout_is(unindent(ALPHA_OUT));
-    scene
-        .ucmd()
-        .arg("--endian=l")
-        .arg("test")
-        .succeeds()
-        .no_stderr()
-        .stdout_is(unindent(ALPHA_OUT));
+
+    for arg in ["--endian=little", "--endian=littl", "--endian=l"] {
+        scene
+            .ucmd()
+            .arg(arg)
+            .arg("test")
+            .succeeds()
+            .stdout_only(unindent(ALPHA_OUT));
+    }
+
     // Ensure that default format matches `-t o2`, and that `-t` does not absorb file argument
     scene
         .ucmd()
         .arg("--endian=little")
         .arg("-t")
         .arg("o2")
-        .arg("test");
+        .arg("test")
+        .succeeds()
+        .stdout_only(unindent(ALPHA_OUT));
 }
 
 // Test that od can read 2 files and concatenate the contents
 #[test]
-fn test_2files() {
+fn test_two_files() {
     let (at, mut ucmd) = at_and_ucmd!();
     at.write("test1", "abcdefghijklmnop");
     at.write("test2", "qrstuvwxyz\n"); // spell-checker:disable-line
@@ -66,15 +61,13 @@ fn test_2files() {
         .arg("test1")
         .arg("test2")
         .succeeds()
-        .no_stderr()
-        .stdout_is(unindent(ALPHA_OUT));
+        .stdout_only(unindent(ALPHA_OUT));
 }
 
 // Test that od gives non-0 exit val for filename that doesn't exist.
 #[test]
-fn test_no_file() {
-    let (_at, mut ucmd) = at_and_ucmd!();
-    ucmd.arg("}surely'none'would'thus'a'file'name").fails();
+fn test_non_existing_file() {
+    new_ucmd!().arg("non_existing_file").fails();
 }
 
 // Test that od reads from stdin instead of a file
@@ -85,8 +78,7 @@ fn test_from_stdin() {
         .arg("--endian=little")
         .run_piped_stdin(input.as_bytes())
         .success()
-        .no_stderr()
-        .stdout_is(unindent(ALPHA_OUT));
+        .stdout_only(unindent(ALPHA_OUT));
 }
 
 // Test that od reads from stdin and also from files
@@ -104,8 +96,7 @@ fn test_from_mixed() {
         .arg("test-3")
         .run_piped_stdin(data2.as_bytes())
         .success()
-        .no_stderr()
-        .stdout_is(unindent(ALPHA_OUT));
+        .stdout_only(unindent(ALPHA_OUT));
 }
 
 #[test]
@@ -116,8 +107,7 @@ fn test_multiple_formats() {
         .arg("-b")
         .run_piped_stdin(input.as_bytes())
         .success()
-        .no_stderr()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             "
             0000000   a   b   c   d   e   f   g   h   i   j   k   l   m   n   o   p
                     141 142 143 144 145 146 147 150 151 152 153 154 155 156 157 160
@@ -145,8 +135,7 @@ fn test_dec() {
         .arg("-s")
         .run_piped_stdin(&input[..])
         .success()
-        .no_stderr()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -165,8 +154,7 @@ fn test_hex16() {
         .arg("-x")
         .run_piped_stdin(&input[..])
         .success()
-        .no_stderr()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -183,8 +171,7 @@ fn test_hex32() {
         .arg("-X")
         .run_piped_stdin(&input[..])
         .success()
-        .no_stderr()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -200,8 +187,8 @@ fn test_f16() {
     ]; // 0x8400 -6.104e-5
     let expected_output = unindent(
         "
-            0000000     1.000         0        -0       inf
-            0000010      -inf       NaN -6.104e-5
+            0000000       1.0000000               0              -0             inf
+            0000010            -inf             NaN   -6.1035156e-5
             0000016
             ",
     );
@@ -211,8 +198,61 @@ fn test_f16() {
         .arg("-w8")
         .run_piped_stdin(&input[..])
         .success()
-        .no_stderr()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
+}
+
+#[test]
+fn test_fh() {
+    let input: [u8; 14] = [
+        0x00, 0x3c, // 0x3C00 1.0
+        0x00, 0x00, // 0x0000 0.0
+        0x00, 0x80, // 0x8000 -0.0
+        0x00, 0x7c, // 0x7C00 Inf
+        0x00, 0xfc, // 0xFC00 -Inf
+        0x00, 0xfe, // 0xFE00 NaN
+        0x00, 0x84,
+    ]; // 0x8400 -6.1035156e-5
+    let expected_output = unindent(
+        "
+            0000000       1.0000000               0              -0             inf
+            0000010            -inf             NaN   -6.1035156e-5
+            0000016
+        ",
+    );
+    new_ucmd!()
+        .arg("--endian=little")
+        .arg("-tfH")
+        .arg("-w8")
+        .run_piped_stdin(&input[..])
+        .success()
+        .stdout_only(expected_output);
+}
+
+#[test]
+fn test_fb() {
+    let input: [u8; 14] = [
+        0x80, 0x3f, // 1.0
+        0x00, 0x00, // 0.0
+        0x00, 0x80, // -0.0
+        0x80, 0x7f, // Inf
+        0x80, 0xff, // -Inf
+        0xc0, 0x7f, // NaN
+        0x80, 0xb8,
+    ]; // -6.1035156e-5
+    let expected_output = unindent(
+        "
+            0000000       1.0000000               0              -0             inf
+            0000010            -inf             NaN   -6.1035156e-5
+            0000016
+        ",
+    );
+    new_ucmd!()
+        .arg("--endian=little")
+        .arg("-tfB")
+        .arg("-w8")
+        .run_piped_stdin(&input[..])
+        .success()
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -228,8 +268,8 @@ fn test_f32() {
     ]; // 0x807f0000 -1.1663108E-38
     let expected_output = unindent(
         "
-            0000000     -1.2345679       12345678  -9.8765427e37             -0
-            0000020            NaN          1e-40 -1.1663108e-38
+            0000000      -1.2345679        12345678  -9.8765427e+37              -0
+            0000020             NaN           1e-40  -1.1663108e-38
             0000034
             ",
     );
@@ -238,8 +278,7 @@ fn test_f32() {
         .arg("-f")
         .run_piped_stdin(&input[..])
         .success()
-        .no_stderr()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -267,24 +306,23 @@ fn test_f64() {
         .arg("-F")
         .run_piped_stdin(&input[..])
         .success()
-        .no_stderr()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
 fn test_multibyte() {
+    let input = "’‐ˆ‘˜語🙂✅🐶𝛑Universität Tübingen \u{1B000}"; // spell-checker:disable-line
     new_ucmd!()
-        .arg("-c")
-        .arg("-w12")
-        .run_piped_stdin("Universität Tübingen \u{1B000}".as_bytes()) // spell-checker:disable-line
+        .args(&["-t", "c"])
+        .run_piped_stdin(input.as_bytes())
         .success()
-        .no_stderr()
-        .stdout_is(unindent(
-            "
-            0000000   U   n   i   v   e   r   s   i   t   ä  **   t
-            0000014       T   ü  **   b   i   n   g   e   n       \u{1B000}
-            0000030  **  **  **
-            0000033
+        .stdout_only(unindent(
+            r"
+            0000000 342 200 231 342 200 220 313 206 342 200 230 313 234 350 252 236
+            0000020 360 237 231 202 342 234 205 360 237 220 266 360 235 233 221   U
+            0000040   n   i   v   e   r   s   i   t 303 244   t       T 303 274   b
+            0000060   i   n   g   e   n     360 233 200 200
+            0000072
             ",
         ));
 }
@@ -305,8 +343,7 @@ fn test_width() {
         .arg("-v")
         .run_piped_stdin(&input[..])
         .success()
-        .no_stderr()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -362,8 +399,7 @@ fn test_width_without_value() {
         .arg("-w")
         .run_piped_stdin(&input[..])
         .success()
-        .no_stderr()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -393,9 +429,8 @@ fn test_suppress_duplicates() {
         .arg("-O")
         .arg("-x")
         .run_piped_stdin(&input[..])
-        .no_stderr()
         .success()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -404,10 +439,10 @@ fn test_big_endian() {
 
     let expected_output = unindent(
         "
-        0000000           -2.0000000000000000
-                    -2.0000000              0
-                      c0000000       00000000
-                   c000   0000    0000   0000
+        0000000             -2.0000000000000000
+                     -2.0000000               0
+                       c0000000        00000000
+                   c000    0000    0000    0000
         0000010
         ",
     );
@@ -419,9 +454,8 @@ fn test_big_endian() {
         .arg("-X")
         .arg("-x")
         .run_piped_stdin(&input[..])
-        .no_stderr()
         .success()
-        .stdout_is(&expected_output);
+        .stdout_only(&expected_output);
     new_ucmd!()
         .arg("--endian=b")
         .arg("-F")
@@ -429,9 +463,8 @@ fn test_big_endian() {
         .arg("-X")
         .arg("-x")
         .run_piped_stdin(&input[..])
-        .no_stderr()
         .success()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -455,9 +488,8 @@ fn test_alignment_Xxa() {
         .arg("-x")
         .arg("-a")
         .run_piped_stdin(&input[..])
-        .no_stderr()
         .success()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -479,9 +511,8 @@ fn test_alignment_Fx() {
         .arg("-F")
         .arg("-x")
         .run_piped_stdin(&input[..])
-        .no_stderr()
         .success()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -507,9 +538,8 @@ fn test_max_uint() {
         .arg("-Dd")
         .arg("--format=u1")
         .run_piped_stdin(&input[..])
-        .no_stderr()
         .success()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -530,9 +560,8 @@ fn test_hex_offset() {
         .arg("-X")
         .arg("-X")
         .run_piped_stdin(&input[..])
-        .no_stderr()
         .success()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -553,9 +582,8 @@ fn test_dec_offset() {
         .arg("-X")
         .arg("-X")
         .run_piped_stdin(&input[..])
-        .no_stderr()
         .success()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -569,9 +597,8 @@ fn test_no_offset() {
         .arg("-X")
         .arg("-X")
         .run_piped_stdin(&input[..])
-        .no_stderr()
         .success()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -607,9 +634,8 @@ fn test_skip_bytes() {
         .arg("-c")
         .arg("--skip-bytes=5")
         .run_piped_stdin(input.as_bytes())
-        .no_stderr()
         .success()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             "
             0000005   f   g   h   i   j   k   l   m   n   o   p   q
             0000021
@@ -624,9 +650,8 @@ fn test_skip_bytes_hex() {
         .arg("-c")
         .arg("--skip-bytes=0xB")
         .run_piped_stdin(input.as_bytes())
-        .no_stderr()
         .success()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             "
             0000013   l   m   n   o   p   q
             0000021
@@ -636,9 +661,8 @@ fn test_skip_bytes_hex() {
         .arg("-c")
         .arg("--skip-bytes=0xE")
         .run_piped_stdin(input.as_bytes())
-        .no_stderr()
         .success()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             "
             0000016   o   p   q
             0000021
@@ -657,14 +681,40 @@ fn test_skip_bytes_error() {
 
 #[test]
 fn test_read_bytes() {
+    let scene = TestScenario::new(util_name!());
+    let fixtures = &scene.fixtures;
+
     let input = "abcdefghijklmnopqrstuvwxyz\n12345678";
-    new_ucmd!()
+
+    fixtures.write("f1", input);
+    let file = fixtures.open("f1");
+    #[cfg(unix)]
+    let mut file_shadow = file.try_clone().unwrap();
+
+    scene
+        .ucmd()
         .arg("--endian=little")
         .arg("--read-bytes=27")
-        .run_piped_stdin(input.as_bytes())
-        .no_stderr()
-        .success()
-        .stdout_is(unindent(ALPHA_OUT));
+        .set_stdin(file)
+        .succeeds()
+        .stdout_only(unindent(ALPHA_OUT));
+
+    // On unix platforms, confirm that only 27 bytes and strictly no more were read from stdin.
+    // Leaving stdin in the correct state is required for GNU compatibility.
+    #[cfg(unix)]
+    {
+        // skip(27) to skip the 27 bytes that should have been consumed with the
+        // --read-bytes flag.
+        let expected_bytes_remaining_in_stdin: Vec<_> = input.bytes().skip(27).collect();
+        let mut bytes_remaining_in_stdin = vec![];
+        assert_eq!(
+            file_shadow
+                .read_to_end(&mut bytes_remaining_in_stdin)
+                .unwrap(),
+            expected_bytes_remaining_in_stdin.len()
+        );
+        assert_eq!(expected_bytes_remaining_in_stdin, bytes_remaining_in_stdin);
+    }
 }
 
 #[test]
@@ -676,16 +726,15 @@ fn test_ascii_dump() {
     new_ucmd!()
         .arg("-tx1zacz")    // spell-checker:disable-line
         .run_piped_stdin(&input[..])
-        .no_stderr()
         .success()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             r"
             0000000  00  01  0a  0d  10  1f  20  61  62  63  7d  7e  7f  80  90  a0  >...... abc}~....<
                     nul soh  nl  cr dle  us  sp   a   b   c   }   ~ del nul dle  sp
-                     \0 001  \n  \r 020 037       a   b   c   }   ~ 177  **  **  **  >...... abc}~....<
+                     \0 001  \n  \r 020 037       a   b   c   }   ~ 177 200 220 240  >...... abc}~....<
             0000020  b0  c0  d0  e0  f0  ff                                          >......<
                       0   @   P   `   p del
-                     ** 300 320 340 360 377                                          >......<
+                    260 300 320 340 360 377                                          >......<
             0000026
             ",
         ));
@@ -704,8 +753,7 @@ fn test_filename_parsing() {
         .arg("--")
         .arg("-f")
         .succeeds()
-        .no_stderr()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             "
             000000   m   i   n   u   s  sp   l   o   w   e   r   c   a   s   e  sp
             000010   f  nl
@@ -721,9 +769,8 @@ fn test_stdin_offset() {
         .arg("-c")
         .arg("+5")
         .run_piped_stdin(input.as_bytes())
-        .no_stderr()
         .success()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             "
             0000005   f   g   h   i   j   k   l   m   n   o   p   q
             0000021
@@ -739,8 +786,7 @@ fn test_file_offset() {
         .arg("-f")
         .arg("10")
         .succeeds()
-        .no_stderr()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             r"
             0000010   w   e   r   c   a   s   e       f  \n
             0000022
@@ -760,9 +806,8 @@ fn test_traditional() {
         .arg("10")
         .arg("0")
         .run_piped_stdin(input.as_bytes())
-        .no_stderr()
         .success()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             r"
             0000010 (0000000)   i   j   k   l   m   n   o   p   q
                                 i   j   k   l   m   n   o   p   q
@@ -781,9 +826,8 @@ fn test_traditional_with_skip_bytes_override() {
         .arg("-c")
         .arg("0")
         .run_piped_stdin(input.as_bytes())
-        .no_stderr()
         .success()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             r"
             0000000   a   b   c   d   e   f   g   h   i   j   k   l   m   n   o   p
             0000020
@@ -800,9 +844,8 @@ fn test_traditional_with_skip_bytes_non_override() {
         .arg("--skip-bytes=10")
         .arg("-c")
         .run_piped_stdin(input.as_bytes())
-        .no_stderr()
         .success()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             r"
             0000012   k   l   m   n   o   p
             0000020
@@ -834,9 +877,8 @@ fn test_traditional_only_label() {
         .arg("10")
         .arg("0x10")
         .run_piped_stdin(input.as_bytes())
-        .no_stderr()
         .success()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             r"
             (0000020)   i   j   k   l   m   n   o   p   q   r   s   t   u   v   w   x
                         i   j   k   l   m   n   o   p   q   r   s   t   u   v   w   x
@@ -868,15 +910,13 @@ fn test_od_invalid_bytes() {
         new_ucmd!()
             .arg(format!("{option}={INVALID_SIZE}"))
             .arg("file")
-            .fails()
-            .code_is(1)
+            .fails_with_code(1)
             .stderr_only(format!("od: invalid {option} argument '{INVALID_SIZE}'\n"));
 
         new_ucmd!()
             .arg(format!("{option}={INVALID_SUFFIX}"))
             .arg("file")
-            .fails()
-            .code_is(1)
+            .fails_with_code(1)
             .stderr_only(format!(
                 "od: invalid suffix in {option} argument '{INVALID_SUFFIX}'\n"
             ));
@@ -884,8 +924,128 @@ fn test_od_invalid_bytes() {
         new_ucmd!()
             .arg(format!("{option}={BIG_SIZE}"))
             .arg("file")
-            .fails()
-            .code_is(1)
+            .fails_with_code(1)
             .stderr_only(format!("od: {option} argument '{BIG_SIZE}' too large\n"));
     }
 }
+
+#[test]
+fn test_od_options_after_filename() {
+    let file = "test";
+    let (at, mut ucmd) = at_and_ucmd!();
+    let input: [u8; 4] = [0x68, 0x1c, 0xbb, 0xfd];
+    at.write_bytes(file, &input);
+
+    ucmd.arg(file)
+        .arg("-v")
+        .arg("-An")
+        .arg("-t")
+        .arg("x2")
+        .succeeds()
+        .stdout_only(" 1c68 fdbb\n");
+}
+
+#[test]
+fn test_od_strings_option() {
+    // Test -S option: output strings of at least N graphic chars
+
+    // Test -S0: output all null-terminated strings regardless of length
+    new_ucmd!()
+        .arg("-S0")
+        .pipe_in(b"hello\x00world\x00")
+        .succeeds()
+        .stdout_is("0000000 hello\n0000006 world\n");
+
+    // Test -S0 with single character strings
+    new_ucmd!()
+        .arg("-S0")
+        .pipe_in(b"a\x00b\x00cd\x00")
+        .succeeds()
+        .stdout_is("0000000 a\n0000002 b\n0000004 cd\n");
+
+    // Test with null-terminated strings
+    new_ucmd!()
+        .arg("-S3")
+        .pipe_in(b"\x01hello\x00world\x00ab\x00")
+        .succeeds()
+        .stdout_is("0000001 hello\n0000007 world\n");
+
+    // Test with -S10 to show only strings with minimum length
+    new_ucmd!()
+        .arg("-S10")
+        .pipe_in(b"\x01          \x00          \x00")
+        .succeeds()
+        .stdout_is("0000001           \n0000014           \n");
+
+    // Test with unterminated string at EOF (should not output)
+    new_ucmd!()
+        .arg("-S10")
+        .pipe_in(b"          ")
+        .succeeds()
+        .no_output();
+
+    // Test with -N limit and pending string (should output even without null)
+    let expected = "0000000   \n";
+    new_ucmd!()
+        .arg("-N2")
+        .arg("-S1")
+        .pipe_in("  ")
+        .succeeds()
+        .stdout_is(expected);
+
+    // Test with -N limit and string too short
+    new_ucmd!()
+        .arg("-N11")
+        .arg("-S11")
+        .pipe_in(b"          \x00")
+        .succeeds()
+        .no_output();
+
+    // Test with no address prefix (-An)
+    new_ucmd!()
+        .arg("-S3")
+        .arg("-An")
+        .pipe_in(b"hello\x00world\x00")
+        .succeeds()
+        .stdout_is("hello\nworld\n");
+}
+
+#[test]
+fn test_od_eintr_handling() {
+    // Test that od properly handles EINTR (ErrorKind::Interrupted) during read operations
+    // This verifies the signal interruption retry logic in PartialReader implementation
+    // This verifies the signal interruption retry logic in PartialReader implementation
+
+    let file = "test_eintr";
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    // Create test data
+    let test_data = [0x48, 0x65, 0x6c, 0x6c, 0x6f, 0x0a]; // "Hello\n"
+    at.write_bytes(file, &test_data);
+
+    // Test that od can handle interrupted reads during file processing
+    // The EINTR handling in PartialReader should retry and complete successfully
+    ucmd.arg(file)
+        .arg("-t")
+        .arg("c")
+        .succeeds()
+        .no_stderr()
+        .stdout_contains("H"); // Should contain 'H' from "Hello"
+
+    // Test with skip and offset options to exercise different code paths
+    // Create a new command instance to avoid "already run this UCommand" error
+    let (at, mut ucmd2) = at_and_ucmd!();
+    at.write_bytes(file, &test_data);
+
+    ucmd2
+        .arg(file)
+        .arg("-j")
+        .arg("1")
+        .arg("-N")
+        .arg("3")
+        .arg("-t")
+        .arg("c")
+        .succeeds()
+        .no_stderr()
+        .stdout_contains("e"); // Should contain 'e' from "ello"
+}
